==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/proto/cproto.cc <==
/*
    interpretation de la syntaxe C
*/
#include "assert.h"
#include <string.h>
#include <lexc.h>
#include "proto.h"
#include "proto.m"

static char regen=0;	/* Reformatte les dclarations */

/*
	ellimine tout jusqu'au point virgule.
	Tient compte des ( et des { imbriqus.
*/
void cproto_waitptv(TOKEN *token)
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNBRACE,	TOK_CLSBRACE},
		{TOK_OPNPAR,		TOK_CLSPAR}
	};
    if (token->type == TOK_OPNBRACE) token_unget(token);
	int c = 0;
	int com_accum = comment_setaccum(0);
    do{
        if (token->type == TOK_PTV) break;
		c++;
		if (c == 100){
			token_inittxt();        /* A ce niveau les tokens sont oublies  */
            	                    /* on peut reallouer le buffer de texte */
                	                /* des tokens                           */
			comment_init();
			c = 0;
		}
    }while (token_getskip(token,tb,2)!=-1);
	comment_setaccum(com_accum);
}

/*
	Analyse de plus haut niveau des noncs
	retourne -1 si fin de fichier, 0 si ok, 1 si } rencontr.
*/
int cproto_line (void)
{
	int ret = -1;
    TOKEN token;

    if (token_get(&token)!= -1){
		static unsigned char extern_C=0;
		ret = 0;
        switch (token.type){
		case TOK_KEYWORD:
			switch (token.type2.keyword){
			case KEY_EXTERN:
				outsrc_prttoken (&token);
				if (token_get (&token) !=-1){
					if (token.type == TOK_STRING){
						if (token_get(&token) != -1){
							if (token.type == TOK_OPNBRACE){
								/* extern "C" { */
								extern_C++;
								cproto_main(regen);
								extern_C--;
							}else{
								cproto_declare(&token,1);
							}
						}
					}else{
						/* Ignore le extern, repousse le token et retraite */
						token_unget(&token);
						ret = cproto_line();
					}
				}else{
					cproto_waitptv  (&token);
				}
				break;
			case KEY_TYPEDEF:
				outsrc_prttoken (&token);
				cproto_waitptv  (&token);
				break;
			default:
				/* declaration */
				cproto_declare (&token,extern_C);
			} 
			break;
		case TOK_ID:
			cproto_declare(&token,extern_C);
			break;
		case TOK_PTV:
			break;
		case TOK_CLSBRACE:
			if (!extern_C){
				erreur (
					"Pas de correspondance pour } ligne %d\n"
					,"No match for } in line %d\n",filesrc_curline());
			}
			ret = 1;
			break;
		default:
			erreur (
				"Syntaxe ligne %d:%d : %s invalide, ignore\n"
				,"Syntax line %d.%d : %s invalid, ignored\n"
				,token.noline,token.nocar,token_txt (&token));
		}
	}
	return ret;
}

void cproto_skippar ()
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNPAR,	TOK_CLSPAR}
	};
	TOKEN token;
    int ret;
    int nol = filesrc_curline();
    while ((ret=token_getskip(&token,tb,1))!= -1 && token.type != TOK_CLSPAR);
    if (ret == -1){
		erreur (MSG_B(E_NOMATCH
			,"Pas de correspondance pour ( ligne %d\n"
			,"No match for ( in line %d\n"),nol);
	}
}

void cproto_skipbrace ()
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNBRACE,	TOK_CLSBRACE},
    };
	TOKEN token;
	int ret;
    int nol = filesrc_curline();
	char *mark = token_getmarktxt();
    while ((ret=token_getskip(&token,tb,1))!=-1 && token.type != TOK_CLSBRACE){
		token_setmarktxt(mark);
    }
    if (ret == -1){
		erreur (
			"Pas de correspondance pour { ligne %d\n"
			,"No match for { in line %d\n",nol);
	}
}
static TOKEN lasttoken;	/* Avant dernier token lue dans la squence pour */
						/* reconnaitre une dclaration de fonction */
						/* ou de classe */

static TOKEN_LIST tokl;	/* Contient un dclaration au complet */
/*
	Essaie de reconnaitre une declaration a partir de regle bidon.

	La fonction dtecte par certaine rgle heuristique que ce n'est
	pas une dclaration de prototype.

	On sait que ce n'est pas une dclaration de classe.

	Si la sequence contient un = apres un TOK_ID, skip ;
	Si la sequence contient un ; erreur
	Si la sequence contient un { apres un TOK_ID, skip ;
	Si la sequence contient un ( apres ), skip ;

	On cherche la sequence TOK_ID OPNPAR
*/ 
static void near cproto_declfct(
	int extern_C)	/* La squence extern "C" devra tre plac */
					/* devant le prototype */
{
	TOKEN token;
	int cplus = 0;			/* Fonction membre C++ dtecte */
	int ret_fct = 0;		/* Fonction retourne pointeur de fonction */
	/* On a rencontre le mot cle operator */
	enum {
		OPERATOR_NIL,	/* Pas encore vue operator */
		OPERATOR_OK,	/* On vient de voir operator */
		OPERATOR_OPNPAR,/* On vient de voir une parenthse qui suit */
						/* operator */
		OPERATOR_CLSPAR,/* On vient de voir "operator()" */
		OPERATOR_AUTRE	/* On a vue un peu n'importe quoi qui peut */
						/* se trouver entre operator et la parenthse */
						/* prcdant les arguments */
	}operator_ok = OPERATOR_NIL;
							
	/*
		Saute la dclaration du type de la fonction jusqu' l'ouverture
		de la parenthse. La parenthse doit tre prcd d'un
		identificateur sauf si on rencontre le mot "operator".
		Si une { ou [ ou ; est rencontr, ce n'est pas une fonction
	*/
	while (1){
		if(token_get(&token)== -1){
            tokrec_prt (&tokl);
            return;
		}
		if (token.type == TOK_SCOPE) cplus = 1;
		if (operator_ok == OPERATOR_OK && token.type == TOK_OPNPAR){
			/* premire parenthse de operator() () */
			operator_ok = OPERATOR_OPNPAR;
		}else if (operator_ok == OPERATOR_OPNPAR){
			if (token.type == TOK_CLSPAR){
				operator_ok = OPERATOR_CLSPAR;
			}else{
				/* Erreur de syntaxe ou pas C++ */
				operator_ok = OPERATOR_NIL;
				token_unget (&token);
				token.type = TOK_OPNPAR;	/* Valide le assert  la fin */
											/* de cette boucle */
				break;
			}
		}else if ((operator_ok == OPERATOR_OK
				|| operator_ok == OPERATOR_AUTRE)
			&& (token.type == TOK_BINOPER
				|| token.type == TOK_UNOPER
				|| token.type == TOK_STAR
				|| token.type == TOK_KEYWORD
				|| token.type == TOK_ID
				|| token.type == TOK_OPNBRAK
				|| token.type == TOK_CLSBRAK)){
			/*
				l'analyse syntaxique ici est un peu tout croche.
				Ca accepte a peu pres n'importe quoi.
				operator + - + ( int ...)
			*/
			operator_ok = OPERATOR_AUTRE;
		}else if ((token.type == TOK_BINOPER
				&& token.type2.binoper != BINOPER_BITAND)
			|| token.type == TOK_OPNBRAK
			|| token.type == TOK_OPNBRACE){
            tokrec_prt (&tokl);
            cproto_waitptv (&token);
			return;
		}else if(token.type == TOK_KEYWORD
			&& token.type2.keyword == KEY_OPERATOR){
			operator_ok = OPERATOR_OK;
		}else if(token.type == TOK_OPNPAR
			&& (lasttoken.type == TOK_ID
				|| lasttoken.type == TOK_KEYWORD
				|| operator_ok == OPERATOR_AUTRE
				|| operator_ok == OPERATOR_CLSPAR)){
			tokrec_add (&tokl,&token);
			break;
		}else if(token.type == TOK_PTV){
            tokrec_prt (&tokl);
            return;
		}else{
			operator_ok  = OPERATOR_NIL;
		}
		tokrec_add (&tokl,&token);
		lasttoken = token;
	}
	assert (token.type == TOK_OPNPAR);
    {
		static TOK_TYPE tb[][2]={
			{TOK_OPNPAR,	TOK_CLSPAR},
			{TOK_OPNBRACE, TOK_CLSBRACE}
		}; 
		if(token_get(&token)== -1){
            tokrec_prt (&tokl);
            return;
		}
		token_unget (&token);
		tokrec_newsub (&tokl);
		tokrec_recskip(&tokl,tb,2);
        tokrec_endsub (&tokl);
		if (token.type == TOK_STAR){
			/* Fonction retournant un pointeur de fonction */
			ret_fct = 1;
			token_get (&token);
			if (token.type == TOK_OPNPAR){
				tokrec_add (&tokl,&token);
				tokrec_newsub (&tokl);
				tokrec_recskip(&tokl,tb,1);
				tokrec_endsub (&tokl);
			}else{
				tokrec_prt (&tokl);
				return;
			}
		}
    }
	token_get (&token);
	if(    token.type == TOK_VIRGULE
		|| token.type == TOK_PTV
		|| (token.type == TOK_BINOPER
			&& token.type2.binoper == BINOPER_AFFECT)
		|| token.type == TOK_OPNPAR){
		tokrec_prt (&tokl);
        outsrc_prttoken(&token);
		cproto_waitptv (&token);
	}else{
		int err = 0;
		int isold = 0;
		if (token.type == TOK_2PT){
			/*
				C'est une dclaration de constructeur C++. C'est
				l'appel des contructeurs des classes drives
			*/
			while (1){
				token_get(&token);
				if (token.type == TOK_OPNBRACE) break;
				outsrc_prttoken (&token);
			}
		}else{
			if (!cplus) isold = cproto_isold (&tokl);
			if (isold==1){
				/*
					Si pas OPN_BRACE, traite vieille declaration
					sinon c'est une declaration nouveau ANSI o les
					types sont int par defaut, ou encore c'est une
					dclaration C++ avec omission du nom de l'argument.

					isold n'est conserve que lorsqu'il y a quelques
					chose entre la ) et {. Sinon ca mlange trop.
				*/
				if (token.type != TOK_OPNBRACE){
					err = cproto_old (&tokl,&token);
				}else{
					isold = 0;
				}
			}else if (token.type == TOK_KEYWORD
				&& token.type2.keyword == KEY_CONST){
				tokrec_add (&tokl,&token);
				token_get (&token);
				if (token.type != TOK_OPNBRACE){
					err = 1;
				}
			}else if (token.type != TOK_OPNBRACE){
				err = 1;
			}
		}
		if (err){
			erreur (
				"Declaration invalide ligne %d '%s'\n"
				,"Invalid declaration on line %d '%s'\n"
				,token_getnoline(),token_txt(&token));
		}else{
			int operator_fct = operator_ok==OPERATOR_CLSPAR;
			cproto_patch(&tokl,&token,ret_fct,operator_fct,isold);
			cproto_new (&tokl,cplus,ret_fct,operator_fct,extern_C);
			if (regen){
				outsrc_print ("\n");
				cproto_format	(&tokl,outsrc_print,ret_fct
					,operator_fct,extern_C);
			}
		}
        outsrc_prttoken (&token);
        cproto_skipbrace();
    }
}
/*
	Essaie de reconnaitre une declaration a partir de regle bidon.

	La fonction dtecte par certaine rgle heuristique que ce n'est
	pas une dclaration de prototype.

	Si c'est une dclaration de class, alors elle donne le controle
	 clsanal.c.

	Si la sequence contient un = apres un TOK_ID, skip ;
	Si la sequence contient un ; erreur
	Si la sequence contient un { apres un TOK_ID, skip ;
	Si la sequence contient un ( apres ), skip ;

	On cherche la sequence TOK_ID OPNPAR
*/ 
void cproto_declare(
	TOKEN *tok,		/* Contient le premier token de la dclaration */
	int extern_C)	/* La squence extern "C" devra tre plac */
					/* devant le prototype */
{
	int is_export = 0;
	tokrec_init (&tokl);
	tokrec_add  (&tokl,tok);
	lasttoken = *tok;
	/*
		Vrifie si c'est un dclaration de classe.
		On attend une sequence simple.

		[export] class TOK_ID :
		On accumule les tokens dans tokl. Si on reconnait pas, on continue
		tout droit.
	*/
	if (lasttoken.type == TOK_KEYWORD
		&& (lasttoken.type2.keyword == KEY_EXPORT
			|| lasttoken.type2.keyword == KEY__EXPORT)){
		token_get(&lasttoken);
		tokrec_add (&tokl,&lasttoken);
		is_export = 1;
	}
	if (lasttoken.type == TOK_KEYWORD
		&& (lasttoken.type2.keyword == KEY_CLASS
			|| lasttoken.type2.keyword == KEY_STRUCT
			|| lasttoken.type2.keyword == KEY_ENUM)){
		KEYWORD keyw = lasttoken.type2.keyword;
		token_get(&lasttoken);
		if (lasttoken.type == TOK_ID){
			/* On attend maintenant le : ou le { */
			/* Dans le cas d'un enum, le : n'est pas possible mais */
			/* ca drange pas */
			TOKEN spc;
			tokrec_add (&tokl,&lasttoken);
			token_get(&spc);
			if (spc.type == TOK_2PT
				|| spc.type == TOK_OPNBRACE){
				tokrec_add (&tokl,&spc);
				tokrec_prt (&tokl);
				if (keyw == KEY_ENUM){
					enumanal_declare (is_export,&lasttoken);
				}else{
					clsanal_declare (is_export,&lasttoken,&spc);
				}
			}else{
				token_unget (&spc);
				cproto_declfct (extern_C);
			}
		}else{
			token_unget (&lasttoken);
			cproto_declfct (extern_C);
		}
	}else{
		cproto_declfct (extern_C);
	}
	tokrec_free 	(&tokl);
}

/*
	Traite un fichier au complet.
	Optionnellement reformatte les dclarations de fonctions.
*/
void cproto_main(int _regen)
{
	regen = (char)_regen;
	while (cproto_line()==0){
        token_inittxt();        /* A ce niveau les tokens sont oublies  */
                                /* on peut reallouer le buffer de texte */
                                /* des tokens                           */
        comment_init();
	}
}

==> linuxconf-tools-1.9.src.tar.gz: linuxconf-tools-1.9/proto/cproto.cc <==
/*
    interpretation de la syntaxe C
*/
#include "assert.h"
#include <string.h>
#include <lexc.h>
#include "proto.h"
#include "proto.m"

static char regen=0;	/* Reformatte les dclarations */

/*
	ellimine tout jusqu'au point virgule.
	Tient compte des ( et des { imbriqus.
*/
void cproto_waitptv(TOKEN *token)
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNBRACE,	TOK_CLSBRACE},
		{TOK_OPNPAR,		TOK_CLSPAR}
	};
    if (token->type == TOK_OPNBRACE) token_unget(token);
	int c = 0;
	int com_accum = comment_setaccum(0);
    do{
        if (token->type == TOK_PTV) break;
		c++;
		if (c == 100){
			token_inittxt();        /* A ce niveau les tokens sont oublies  */
            	                    /* on peut reallouer le buffer de texte */
                	                /* des tokens                           */
			comment_init();
			c = 0;
		}
    }while (token_getskip(token,tb,2)!=-1);
	comment_setaccum(com_accum);
}

/*
	Analyse de plus haut niveau des noncs
	retourne -1 si fin de fichier, 0 si ok, 1 si } rencontr.
*/
int cproto_line (void)
{
	int ret = -1;
    TOKEN token;

    if (token_get(&token)!= -1){
		static unsigned char extern_C=0;
		ret = 0;
        switch (token.type){
		case TOK_KEYWORD:
			switch (token.type2.keyword){
			case KEY_EXTERN:
				outsrc_prttoken (&token);
				if (token_get (&token) !=-1){
					if (token.type == TOK_STRING){
						if (token_get(&token) != -1){
							if (token.type == TOK_OPNBRACE){
								/* extern "C" { */
								extern_C++;
								cproto_main(regen);
								extern_C--;
							}else{
								cproto_declare(&token,1);
							}
						}
					}else{
						/* Ignore le extern, repousse le token et retraite */
						token_unget(&token);
						ret = cproto_line();
					}
				}else{
					cproto_waitptv  (&token);
				}
				break;
			case KEY_TYPEDEF:
				outsrc_prttoken (&token);
				cproto_waitptv  (&token);
				break;
			default:
				/* declaration */
				cproto_declare (&token,extern_C);
			} 
			break;
		case TOK_ID:
			cproto_declare(&token,extern_C);
			break;
		case TOK_PTV:
			break;
		case TOK_CLSBRACE:
			if (!extern_C){
				erreur (
					"Pas de correspondance pour } ligne %d\n"
					,"No match for } in line %d\n",filesrc_curline());
			}
			ret = 1;
			break;
		default:
			erreur (
				MSG_B(E_CSYNTAX
					,"Syntaxe ligne %d:%d : %s invalide, ignore\n"
					,"Syntax line %d.%d : %s invalid, ignored\n")
				,token.noline,token.nocar,token_txt (&token));
		}
	}
	return ret;
}

void cproto_skippar ()
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNPAR,	TOK_CLSPAR}
	};
	TOKEN token;
    int ret;
    int nol = filesrc_curline();
    while ((ret=token_getskip(&token,tb,1))!= -1 && token.type != TOK_CLSPAR);
    if (ret == -1){
		erreur (MSG_B(E_NOMATCH
			,"Pas de correspondance pour ( ligne %d\n"
			,"No match for ( in line %d\n"),nol);
	}
}

void cproto_skipbrace ()
{
	static TOK_TYPE tb[][2]={
		{TOK_OPNBRACE,	TOK_CLSBRACE},
    };
	TOKEN token;
	int ret;
    int nol = filesrc_curline();
	char *mark = token_getmarktxt();
    while ((ret=token_getskip(&token,tb,1))!=-1 && token.type != TOK_CLSBRACE){
		token_setmarktxt(mark);
    }
    if (ret == -1){
		erreur (
			"Pas de correspondance pour { ligne %d\n"
			,"No match for { in line %d\n",nol);
	}
}
static TOKEN lasttoken;	/* Avant dernier token lue dans la squence pour */
						/* reconnaitre une dclaration de fonction */
						/* ou de classe */

static TOKEN_LIST tokl;	/* Contient un dclaration au complet */
/*
	Essaie de reconnaitre une declaration a partir de regle bidon.

	La fonction dtecte par certaine rgle heuristique que ce n'est
	pas une dclaration de prototype.

	On sait que ce n'est pas une dclaration de classe.

	Si la sequence contient un = apres un TOK_ID, skip ;
	Si la sequence contient un ; erreur
	Si la sequence contient un { apres un TOK_ID, skip ;
	Si la sequence contient un ( apres ), skip ;

	On cherche la sequence TOK_ID OPNPAR
*/ 
static void near cproto_declfct(
	int extern_C)	/* La squence extern "C" devra tre plac */
					/* devant le prototype */
{
	TOKEN token;
	int cplus = 0;			/* Fonction membre C++ dtecte */
	int ret_fct = 0;		/* Fonction retourne pointeur de fonction */
	/* On a rencontre le mot cle operator */
	enum {
		OPERATOR_NIL,	/* Pas encore vue operator */
		OPERATOR_OK,	/* On vient de voir operator */
		OPERATOR_OPNPAR,/* On vient de voir une parenthse qui suit */
						/* operator */
		OPERATOR_CLSPAR,/* On vient de voir "operator()" */
		OPERATOR_AUTRE	/* On a vue un peu n'importe quoi qui peut */
						/* se trouver entre operator et la parenthse */
						/* prcdant les arguments */
	}operator_ok = OPERATOR_NIL;
							
	/*
		Saute la dclaration du type de la fonction jusqu' l'ouverture
		de la parenthse. La parenthse doit tre prcd d'un
		identificateur sauf si on rencontre le mot "operator".
		Si une { ou [ ou ; est rencontr, ce n'est pas une fonction
	*/
	while (1){
		if(token_get(&token)== -1){
            tokrec_prt (&tokl);
            return;
		}
		if (token.type == TOK_SCOPE) cplus = 1;
		if (operator_ok == OPERATOR_OK && token.type == TOK_OPNPAR){
			/* premire parenthse de operator() () */
			operator_ok = OPERATOR_OPNPAR;
		}else if (operator_ok == OPERATOR_OPNPAR){
			if (token.type == TOK_CLSPAR){
				operator_ok = OPERATOR_CLSPAR;
			}else{
				/* Erreur de syntaxe ou pas C++ */
				operator_ok = OPERATOR_NIL;
				token_unget (&token);
				token.type = TOK_OPNPAR;	/* Valide le assert  la fin */
											/* de cette boucle */
				break;
			}
		}else if ((operator_ok == OPERATOR_OK
				|| operator_ok == OPERATOR_AUTRE)
			&& (token.type == TOK_BINOPER
				|| token.type == TOK_UNOPER
				|| token.type == TOK_STAR
				|| token.type == TOK_KEYWORD
				|| token.type == TOK_ID
				|| token.type == TOK_OPNBRAK
				|| token.type == TOK_CLSBRAK)){
			/*
				l'analyse syntaxique ici est un peu tout croche.
				Ca accepte a peu pres n'importe quoi.
				operator + - + ( int ...)
			*/
			operator_ok = OPERATOR_AUTRE;
		}else if ((token.type == TOK_BINOPER
				&& token.type2.binoper != BINOPER_BITAND)
			|| token.type == TOK_OPNBRAK
			|| token.type == TOK_OPNBRACE){
            tokrec_prt (&tokl);
            cproto_waitptv (&token);
			return;
		}else if(token.type == TOK_KEYWORD
			&& token.type2.keyword == KEY_OPERATOR){
			operator_ok = OPERATOR_OK;
		}else if(token.type == TOK_OPNPAR
			&& (lasttoken.type == TOK_ID
				|| lasttoken.type == TOK_KEYWORD
				|| operator_ok == OPERATOR_AUTRE
				|| operator_ok == OPERATOR_CLSPAR)){
			tokrec_add (&tokl,&token);
			break;
		}else if(token.type == TOK_PTV){
            tokrec_prt (&tokl);
            return;
		}else{
			operator_ok  = OPERATOR_NIL;
		}
		tokrec_add (&tokl,&token);
		lasttoken = token;
	}
	assert (token.type == TOK_OPNPAR);
    {
		static TOK_TYPE tb[][2]={
			{TOK_OPNPAR,	TOK_CLSPAR},
			{TOK_OPNBRACE, TOK_CLSBRACE}
		}; 
		if(token_get(&token)== -1){
            tokrec_prt (&tokl);
            return;
		}
		token_unget (&token);
		tokrec_newsub (&tokl);
		tokrec_recskip(&tokl,tb,2);
        tokrec_endsub (&tokl);
		if (token.type == TOK_STAR){
			/* Fonction retournant un pointeur de fonction */
			ret_fct = 1;
			token_get (&token);
			if (token.type == TOK_OPNPAR){
				tokrec_add (&tokl,&token);
				tokrec_newsub (&tokl);
				tokrec_recskip(&tokl,tb,1);
				tokrec_endsub (&tokl);
			}else{
				tokrec_prt (&tokl);
				return;
			}
		}
    }
	token_get (&token);
	if(    token.type == TOK_VIRGULE
		|| token.type == TOK_PTV
		|| (token.type == TOK_BINOPER
			&& token.type2.binoper == BINOPER_AFFECT)
		|| token.type == TOK_OPNPAR){
		tokrec_prt (&tokl);
        outsrc_prttoken(&token);
		cproto_waitptv (&token);
	}else{
		int err = 0;
		int isold = 0;
		if (token.type == TOK_2PT){
			/*
				C'est une dclaration de constructeur C++. C'est
				l'appel des contructeurs des classes drives
			*/
			while (1){
				token_get(&token);
				if (token.type == TOK_OPNBRACE) break;
				outsrc_prttoken (&token);
			}
		}else{
			if (!cplus) isold = cproto_isold (&tokl);
			if (isold==1){
				/*
					Si pas OPN_BRACE, traite vieille declaration
					sinon c'est une declaration nouveau ANSI o les
					types sont int par defaut, ou encore c'est une
					dclaration C++ avec omission du nom de l'argument.

					isold n'est conserve que lorsqu'il y a quelques
					chose entre la ) et {. Sinon ca mlange trop.
				*/
				if (token.type != TOK_OPNBRACE){
					err = cproto_old (&tokl,&token);
				}else{
					isold = 0;
				}
			}else if (token.type == TOK_KEYWORD
				&& token.type2.keyword == KEY_CONST){
				tokrec_add (&tokl,&token);
				token_get (&token);
				if (token.type != TOK_OPNBRACE){
					err = 1;
				}
			}else if (token.type != TOK_OPNBRACE){
				err = 1;
			}
		}
		if (err){
			erreur (
				"Declaration invalide ligne %d '%s'\n"
				,"Invalid declaration on line %d '%s'\n"
				,token_getnoline(),token_txt(&token));
		}else{
			int operator_fct = operator_ok==OPERATOR_CLSPAR;
			cproto_patch(&tokl,&token,ret_fct,operator_fct,isold);
			cproto_new (&tokl,cplus,ret_fct,operator_fct,extern_C);
			if (regen){
				outsrc_print ("\n");
				cproto_format	(&tokl,outsrc_print,ret_fct
					,operator_fct,extern_C);
			}
		}
        outsrc_prttoken (&token);
        cproto_skipbrace();
    }
}
/*
	Essaie de reconnaitre une declaration a partir de regle bidon.

	La fonction dtecte par certaine rgle heuristique que ce n'est
	pas une dclaration de prototype.

	Si c'est une dclaration de class, alors elle donne le controle
	 clsanal.c.

	Si la sequence contient un = apres un TOK_ID, skip ;
	Si la sequence contient un ; erreur
	Si la sequence contient un { apres un TOK_ID, skip ;
	Si la sequence contient un ( apres ), skip ;

	On cherche la sequence TOK_ID OPNPAR
*/ 
void cproto_declare(
	TOKEN *tok,		/* Contient le premier token de la dclaration */
	int extern_C)	/* La squence extern "C" devra tre plac */
					/* devant le prototype */
{
	int is_export = 0;
	tokrec_init (&tokl);
	tokrec_add  (&tokl,tok);
	lasttoken = *tok;
	/*
		Vrifie si c'est un dclaration de classe.
		On attend une sequence simple.

		[export] class TOK_ID :
		On accumule les tokens dans tokl. Si on reconnait pas, on continue
		tout droit.
	*/
	if (lasttoken.type == TOK_KEYWORD
		&& (lasttoken.type2.keyword == KEY_EXPORT
			|| lasttoken.type2.keyword == KEY__EXPORT)){
		token_get(&lasttoken);
		tokrec_add (&tokl,&lasttoken);
		is_export = 1;
	}
	if (lasttoken.type == TOK_KEYWORD
		&& (lasttoken.type2.keyword == KEY_CLASS
			|| lasttoken.type2.keyword == KEY_STRUCT
			|| lasttoken.type2.keyword == KEY_ENUM)){
		KEYWORD keyw = lasttoken.type2.keyword;
		token_get(&lasttoken);
		if (lasttoken.type == TOK_ID){
			/* On attend maintenant le : ou le { */
			/* Dans le cas d'un enum, le : n'est pas possible mais */
			/* ca drange pas */
			TOKEN spc;
			tokrec_add (&tokl,&lasttoken);
			token_get(&spc);
			if (spc.type == TOK_2PT
				|| spc.type == TOK_OPNBRACE){
				tokrec_add (&tokl,&spc);
				tokrec_prt (&tokl);
				if (keyw == KEY_ENUM){
					enumanal_declare (is_export,&lasttoken);
				}else{
					clsanal_declare (is_export,&lasttoken,&spc);
				}
			}else{
				token_unget (&spc);
				cproto_declfct (extern_C);
			}
		}else{
			token_unget (&lasttoken);
			cproto_declfct (extern_C);
		}
	}else{
		cproto_declfct (extern_C);
	}
	tokrec_free 	(&tokl);
}

/*
	Traite un fichier au complet.
	Optionnellement reformatte les dclarations de fonctions.
*/
void cproto_main(int _regen)
{
	regen = (char)_regen;
	while (cproto_line()==0){
        token_inittxt();        /* A ce niveau les tokens sont oublies  */
                                /* on peut reallouer le buffer de texte */
                                /* des tokens                           */
        comment_init();
	}
}
